/*
   This prettyPrint() method takes a Tree
   and converts it into a well formated string.

   This pretty-printer does the following.
      1) It puts the left node "inline" with the root.
      2) It handles the special case when depth=1.
      3) It handles the special case when depth=2.
      4) Closing parentheses are either aligned vertically
         or are all on the end of a line.
*/

public class PrettyPrinter
{
   public static boolean ALIGN = false;

   public static String prettyPrint(Tree tree)
   {
      return prettyPrint(tree, 0);
   }

   public static String prettyPrint(Tree tree, int indentation)
   {
      String result = "";

      if ( 0 == tree.degree() )
      {
         result += tree.getElement();
      }
      else if ( (1 == tree.depth()) || (2 == tree.depth()) )
      {
         result += "(" + tree.getElement();

         for (int i = 0; i < tree.degree(); i++)
            result += " " + prettyPrint( tree.getSubTree(i), indentation );

         result += ")";
      }
      else
      {
         result += "(" + tree.getElement();

         indentation += 2 + tree.getElement().length();
         String indent = "";
         for (int i = 0; i < indentation; i++)
            indent += " ";

         // First branch of this tree.
         result += " " + prettyPrint( tree.getSubTree(0), indentation );
         // The rest of this tree.
         for (int i = 1; i < tree.degree(); i++)
            result += "\n" + indent + prettyPrint( tree.getSubTree(i), indentation );

         result += ALIGN ? ("\n" + indent + ")") : ")";
      }
      return result;
   }
}//PrettyPrinter6
